def good_to_go():
    """ Checks that OPEN_AI_KEY_0 etc are in env
    :return:
    """
    import os
    try:
        from gptprobe.private_setenv import NOTHING
    except ImportError:
        pass
    checks  = [ os.environ.get(f'OPEN_AI_KEY_{key_choice}') for key_choice in range(3) ]
    return all(checks)


import re
import json


def parse_dict_robustly(text):
    # Search for dictionary-like substrings within the input text
    dict_pattern = r"[{\[].*?[}\]]"
    dict_matches = re.findall(dict_pattern, text)

    if not dict_matches:
        return None

    kv_dict = {}

    for dict_string in dict_matches:
        # Extract substrings that could be key-value pairs
        kv_pattern = r'("[^"]+"\s*:\s*-?\d+(\.\d+)?([eE][-+]?\d+)?)'
        kv_matches = re.findall(kv_pattern, dict_string)

        if not kv_matches:
            continue

        for match in kv_matches:
            # Wrap the key-value pair in curly braces to form a valid JSON string
            kv_json_string = '{' + match[0] + '}'

            try:
                kv_pair = json.loads(kv_json_string)
                kv_dict.update(kv_pair)
            except json.JSONDecodeError:
                continue

    return kv_dict if kv_dict else None




import re
import json


def parse_dict_robustly_singular(text):
    # Search for a dictionary-like substring within the input text
    dict_pattern = r"[{\[].*?[}\]]"
    dict_match = re.search(dict_pattern, text)

    if not dict_match:
        return None

    dict_string = dict_match.group(0)

    # Extract substrings that could be key-value pairs
    kv_pattern = r'("[^"]+"\s*:\s*-?\d+(\.\d+)?([eE][-+]?\d+)?)'
    kv_matches = re.findall(kv_pattern, dict_string)

    if not kv_matches:
        return None

    kv_dict = {}
    for match in kv_matches:
        # Wrap the key-value pair in curly braces to form a valid JSON string
        if len(match):
            kv_json_string = '{' + match[0] + '}'

            try:
                kv_pair = json.loads(kv_json_string)
                kv_dict.update(kv_pair)
            except json.JSONDecodeError:
                continue

    return kv_dict if kv_dict else None



def dict_equal_or_none(dict1, dict2):
    # Check if both dictionaries have the same keys, or both are none

    if not dict1:
        return not dict2

    if not dict2:
        return not dict1

    if set(dict1.keys()) != set(dict2.keys()):
        return False

    # Compare the values for each key
    for key in dict1:
        if dict1[key] != dict2[key]:
            return False

    return True


if __name__=='__main__':
    text = 'Some unstructured text {"key1": 1.23, "key2": 45.6}, more text {"key3": 7.89} and so on.'
    result = parse_dict_robustly(text)
    print(result)



from gptprobe.openaiwrappers.openapi import ask_using_environ


def ask_for_numeric_dict(text:str, key_choice=0, **open_kwargs):
    """ Try to turn any text into a dictionary
    :param text:         Unstructured text you hope implies a dictionary with numeric values
    :param key_choice:   References an env variable like  OPEN_AI_KEY_0
    :return:
    """
    question = """I am going to give you unstructured text delineated in xml-style as <text>...</text> and I would like you to return
     a dictionary with numeric keys, if it is at all possible to interpret the text that way, otherwise return an empty dictionary like '{}'. 
     If there appear to be extraneous lines in the response, just ignore them. 
     Do not respond with any other text. Here is the unstructured text <text>"""+text+"""</text>"""
    response = ask_using_environ(question, key_choice=key_choice, **open_kwargs)
    return response


if __name__=='__main__':
    import json
    from gptprobe.private_setenv import NOTHING
    from pprint import pprint

    text = """ dog: 1563,  sydney: 1
               cow: 1765"""
    d3_txt = ask_for_numeric_dict(text=text,engine='text-davinci-003', temperature=0.2)
    d2_txt = ask_for_numeric_dict(text=text, engine='text-davinci-003')
    pprint({'d3':d3_txt,
            'd2':d2_txt})
    from gptprobe.parsing.parsedictrobustly import dict_equal_or_none, parse_dict_robustly
    d3 = parse_dict_robustly(d3_txt)
    d2 = parse_dict_robustly(d2_txt)
    dict_equal_or_none(d2, d3)



import openai
import os
from gptprobe.openaiwrappers.opendefaults import DEFAULT_ENGINE, DEFAULT_TEMPERATURE, DEFAULT_MAX_TOKENS


def ask_using_environ(question, key_choice:int=0,
                      engine=None,
                      max_tokens=None,
                      temperature=None, **kwargs):
    """ Use open ai keys in environment variables like OPEN_AI_KEY_0 to ask a question
    :param key_choice  0, 1, or 2  typically
    :return:
    """
    if engine is None:
        engine = os.environ.get('OPEN_AI_ENGINE') or DEFAULT_ENGINE

    if temperature is None:
        try:
            temperature = float(os.environ.get('OPEN_AI_TEMPERATURE'))
        except TypeError:
            temperature = DEFAULT_TEMPERATURE

    if max_tokens is None:
        try:
            max_tokens = int(os.environ.get('OPEN_AI_MAX_TOKENS'))
        except TypeError:
            max_tokens = DEFAULT_MAX_TOKENS


    # Creds
    api_env = f"OPEN_AI_KEY_{key_choice}"
    api_key = os.environ.get(api_env)
    if api_key is None:
        raise ValueError(f'The environment variable {api_env} must be set)')
    openai.api_key = api_key

    response = openai.Completion.create(
        engine=engine,
        prompt=question,
        max_tokens=max_tokens,
        stop=None,
        temperature=temperature,
        **kwargs
    )
    return response.choices[0].text.strip()


if __name__=='__main__':
    from gptprobe.private_setenv import NOTHING
    print(ask_using_environ(question="Is this a good question?"))


# Just an example of setting the env variables required to use this package.

GPT_PROBE_0 = 'sk-ekOvFjAHKETQ9NOTAREALKEYSDNXwE3jyy7IHKA4s'
GPT_PROBE_1 = 'sk-X0hK82TSSbananasinpijamasarerunning downz1gpT3'
GPT_PROBE_2 = 'sk-24W74IputyourkeyshereobviouslyQyQFdt2ymTewRc'
import os

os.environ['OPEN_AI_KEY_0'] = GPT_PROBE_0
os.environ['OPEN_AI_KEY_1'] = GPT_PROBE_1
os.environ['OPEN_AI_KEY_2'] = GPT_PROBE_2

NOTHING ='nothing'



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="gptprobe",
    version="0.0.5",
    description="Probing chatgpt",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/gptprobe/gptprobe",
    author="gptprobe",
    author_email="pcotton@intechinvestments.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    packages=["gptprobe","gptprobe.openaiwrappers","gptprobe.utilfortests","gptprobe.parsing"],
    test_suite='pytest',
    tests_require=['pytest'],
    include_package_data=True,
    install_requires=["numpy","pandas","getjson","openai"],
    entry_points={
        "console_scripts": [
            "gptprobe=gptprobe.__main__:main",
        ]
    },
)
